import sys

from .process import workspace
from .process import split, annotate, save


def main() -> int:
    if len(sys.argv) == 1:
        return -1

    args = sys.argv[1:]
    method = args[0]

    if method == '--init':
        workspace.init()

    elif method == '--reset':
        workspace.clean()

    elif method == '--split':
        split()
        annotate()

    elif method == '--annotate':
        annotate()

    elif method == '--save':
        save()

    elif method == '--reset':
        workspace.clean()

    else:
        print(f'"{method}" is not a valid command.')

    return 0



import os

from .workspace import WORKSPACE
from ..utils.filesystem import load_data, \
                               append_data


def save() -> None:
    dataset = set(load_data(os.path.join(WORKSPACE, '2', 'dev-ents.txt')))
    append_data(os.path.join(WORKSPACE, '4', 'ents.txt'), dataset)

    redacted_dataset = set(load_data(os.path.join(WORKSPACE, '3', 'dev-ents-redacted.txt')))
    append_data(os.path.join(WORKSPACE, '4', 'ents-redacted.txt'), redacted_dataset)



from typing import List, Callable, Generator, Iterator, cast
from extr import Entity, TokenGroup
from extr.entities import EntityExtractor
from extr.tokenizers import tokenizer
from ..models import Label


class IOB():
    def __init__(self, sentence_tokenizer: Callable[[str], List[List[str]]], entity_extractor: EntityExtractor) -> None:
        self._sentence_tokenizer = sentence_tokenizer
        self._entity_extractor = entity_extractor

    def label(self, text: str) -> Generator[Label, None, None]:
        entities = self._entity_extractor.get_entities(text)
        token_groups = tokenizer(text, self._sentence_tokenizer(text))

        return self.__merge(token_groups, entities)

    def __merge(self, token_groups: Generator[TokenGroup, None, None], entities: List[Entity]) -> Generator[Label, None, None]:
        for token_group in token_groups:
            tokens = list(enumerate(token_group.tokens))
            labels = ['O' for _ in range(len(tokens))]
            for entity in cast(Iterator[Entity], filter(token_group.contains, entities)):
                used_counter = 0

                for i, token in tokens:
                    if not entity.is_in(token) and not entity.contains(token):
                        continue

                    current_label = labels[i]
                    assert current_label == 'O', f'bad tokenizer? multiple entities belong to the same token - {current_label}'

                    labels[i] = 'B-' if used_counter == 0 else 'I-'
                    labels[i] += entity.label

                    token.add_entity(entity)

                    used_counter += 1

            yield Label(token_group.tokens, labels)



import os
import random

from .workspace import load_config, WORKSPACE
from ..utils.filesystem import load_data, \
                               save_data


def split() -> None:
    config = load_config()

    split_config = config['split'] if 'split' in config else {}
    seed = split_config['seed'] if 'seed' in split_config else None
    partition_point = split_config['amount'] if 'amount' in split_config else 25

    source_data = load_data(
        os.path.join(WORKSPACE, '1', config['source'])
    )

    if seed:
        random.seed(seed)

    random.shuffle(source_data)

    pivot = int(len(source_data) * partition_point) if partition_point < 1 else partition_point
    development_set, holdout_set = (source_data[:pivot], source_data[pivot:])

    output_directory = os.path.join(WORKSPACE, '2')
    save_data(os.path.join(output_directory, 'dev.txt'), development_set)
    save_data(os.path.join(output_directory, 'holdouts.txt'), holdout_set)
